package json.serialize;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * 
 * This class is responsible for serializing a java object into JSON text.
 * It can serialize pure objects which are representative of native types,
 * such as <code>Boolean</code>, <code>String</code> and <code>Double</code>. 
 * It can also serialize arrays and collections of these objects such as
 * <code>ArrayList</code> and <code>HashMap</code>. It can also serialize composite objects
 * of these types through public fields and bean access methods. 
 * <br>
 * Usage: The object is serialized to a <code>String</code> through the <code>serialize()</code>
 * function.
 * <br>
 *
 */
public class JSONSerializer {
	
	/**
	 * Constant String
	 */
	private static final String ASSIGN_ELEMENT=":";
	
	/**
	 * Constant String
	 */
	private static final String NEXT_ELEMENT=",";
	
	/**
	 * Constant String
	 */
	private static final String START_OBJECT="{";
	
	/**
	 * Constant String
	 */
	private static final String END_OBJECT="}";
	
	/**
	 * Constant String
	 */
	private static final String START_ARRAY="[";
	
	/**
	 * Constant String 
	 */
	private static final String END_ARRAY="]";
	
	/**
	 * Constant String
	 */
	private static final String START_STRING="\"";
	
	/**
	 * Constant String
	 */
	private static final String END_STRING="\"";
	
	/**
	 * Constant String
	 */
	private static final String JS_NULL="null";
	
	/**
	 * Constant String
	 */
	private static final String JS_VAR="var";
	
	/**
	 * Constant String
	 */
	private static final String JS_VARSPCSTART="u";
	
	/**
	 * Constant String
	 */
	private static final String JS_HEXPAD="0000";
	
	/**
	 * Counter used for creating unique variable names 
	 * for unnamed variables
	 */
	private int vCounter=0;

	/**
	 * Converts a number into a JS Hex Number
	 * 
	 * @param pNumber Number to ass
	 * @return A String with the formatted hexadecimal equivalent
	 */
	private String jsHex(Number pNumber){
		StringBuffer mStrb=new StringBuffer();
		mStrb.append(JS_HEXPAD);
		mStrb.append(Long.toHexString(pNumber.longValue()));
		return mStrb.substring(mStrb.length()-JS_HEXPAD.length());
	}

	/**
	 * Converts a String into a JSON representable format
	 * 
	 * @param pOrig A String
	 * @return A JS-escaped version of the String
	 */
	private String jsQuoteString(String pOrig){
		StringBuffer mStrb=new StringBuffer();
		int mIter=0;
		char mChar;
		if(pOrig==null||"".equals(pOrig)) return "";
		for(;mIter<pOrig.length();++mIter){
			mChar=pOrig.charAt(mIter);
			if(mChar=='\"') {
				mStrb.append("\\\"");
			} else if(mChar=='\\'){
				mStrb.append("\\\\");
			} else if(mChar=='/'){
				mStrb.append("\\/");
			} else if(mChar=='\b'){
				mStrb.append("\\b");
			} else if(mChar=='\f'){
				mStrb.append("\\f");
			} else if(mChar=='\n'){
				mStrb.append("\\n");
			} else if(mChar=='\r'){
				mStrb.append("\\r");
			} else if(mChar=='\t'){
				mStrb.append("\\t");
			} else if(Character.isLetterOrDigit((int)mChar)&&mChar<256){
				mStrb.append(mChar);
			} else {
				mStrb.append("\\u");
				mStrb.append(jsHex(new Integer((int)mChar)));
			}
		}
		return mStrb.toString();
	}

	/**
	 * Converts a String into a JSON variable representable format
	 * 
	 * @param pOrig A String
	 * @return A JS-friendly variable name
	 */
	private String jsVarQuoteString(String pOrig){
		StringBuffer mStrb=new StringBuffer();
		int mIter=0;
		char mChar;
		if(pOrig==null||"".equals(pOrig)) return "";
		for(;mIter<pOrig.length();++mIter){			
			mChar=pOrig.charAt(mIter);
			if(mIter==0&&Character.isDigit((int)mChar)){
				mStrb.append(JS_VAR);
				mStrb.append(mChar);
			} else {
				if(Character.isLetterOrDigit((int)mChar)&&mChar<256){
					mStrb.append(mChar);
				} else {
					mStrb.append(JS_VARSPCSTART);
					mStrb.append(jsHex(new Integer((int)mChar)));
				}
			}
		}
		return mStrb.toString();		
	}

	/**
	 * This function checks if an object is representative of primitive
	 * data types, such as <code>Boolean</code> ad <code>String</code>.
	 * 
	 * @param pObj The Object to be checked
	 * @return true if the object is primitive, false if not
	 */
	private boolean isPrimitive(Object pObj) {
		if(pObj==null) return false;
		if(pObj instanceof Boolean) return true;
		if(pObj instanceof Byte) return true;
		if(pObj instanceof Character) return true;
		if(pObj instanceof Double) return true;
		if(pObj instanceof Enum) return true;
		if(pObj instanceof Float) return true;
		if(pObj instanceof Integer) return true;
		if(pObj instanceof Long) return true;
		if(pObj instanceof Number) return true;
		if(pObj instanceof String) return true;
		if(pObj instanceof Void) return true;
		return false;	
	}

	/**
	 * This function checks if an object is valid for serializable
	 * or not. This is different from classes which implement the
	 * <code>Serializable</code> interface.
	 * 
	 * @param pObj The Object to be checked
	 * @return true if the object is serialzable, false if not
	 */
	private boolean isSerializable(Object pObj) {
		if(pObj==null) return false;		
		if(pObj instanceof Type) return false;		
		return true;
	}

	/**
	 * This function checks if an object is an array of primitive
	 * objects. It doesn't work for arrays of primitives. While
	 * <code>Integer[]</code> will work, <code>int[]</code> won't work.  
	 * 
	 * @param pObj The Object to be checked
	 * @return true if the object is an array, false if not
	 */
	private boolean isArray(Object pObj) {
		if(pObj==null) return false;		
		if(pObj instanceof Object[]) return true;
		return false;
	}

	/**
	 * Serializes any <code>Object</code> into JSON <code>null</code>   
	 * 
	 * @param pObj The Object to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeNull(Object pObj) {
		return JS_NULL;
	}	
	
	/**
	 * Serializes a <code>Boolean</code> object into JSON   
	 * 
	 * @param pBool The Boolean to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeBoolean(Boolean pBool) {
		return pBool.toString();
	}
	
	/**
	 * Serializes a <code>Byte</code> object into JSON   
	 * 
	 * @param pByte The Byte to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeByte(Byte pByte) {
		return pByte.toString();
	}
	
	/**
	 * Serializes a <code>Character</code> object into JSON   
	 * 
	 * @param pChar The Character to be JSON serialized
	 * @return The JSON representation as a Quoted String
	 */	
	private String serializeCharacter(Character pChar) {
		return serializeString(pChar.toString());
	}
		
	/**
	 * Serializes a <code>Double</code> object into JSON   
	 * 
	 * @param pDouble The Double to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeDouble(Double pDouble) {
		return serializeNumber(pDouble);
	}
	
	/**
	 * Serializes an <code>Enum</code> object into JSON   
	 * 
	 * @param pEnum The Enum to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeEnum(Enum pEnum) {
		return serializeString(pEnum.toString());
	}	
	
	/**
	 * Serializes a <code>Float</code> object into JSON   
	 * 
	 * @param pFloat The Float to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeFloat(Float pFloat) {
		return serializeNumber(pFloat);
	}
	
	/**
	 * Serializes an <code>Integer</code> object into JSON   
	 * 
	 * @param pInt The Integer to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeInteger (Integer pInt) {
		return serializeNumber(pInt);
	}

	/**
	 * Serializes a <code>Long</code> object into JSON   
	 * 
	 * @param pLong The Long to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeLong(Long pLong) {
		return serializeNumber(pLong);
	}

	/**
	 * Serializes a <code>Number</code> object into JSON   
	 * 
	 * @param pNumber The Number to be JSON serialized
	 * @return The JSON representation as a String
	 */	
	private String serializeNumber(Number pNumber) {
		return pNumber.toString();
	}
	
	/**
	 * Serializes a <code>String</code> object into JSON   
	 * 
	 * @param pStr The String to be JSON serialized
	 * @return The JSON representation as a Quoted String
	 */	
	private String serializeString(String pStr) {
		StringBuffer mStrb=new StringBuffer();
		mStrb.append(START_STRING);
		mStrb.append(jsQuoteString(pStr));
		mStrb.append(END_STRING);				
		return mStrb.toString();
	}	

	/**
	 * Serializes a <code>Short</code> object into JSON   
	 * 
	 * @param pShort The Short to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeShort(Short pShort) {
		return serializeNumber(pShort);
	}
	
	/**
	 * Serializes a <code>Void</code> object into JSON   
	 * 
	 * @param pVoid The Void to be JSON serialized
	 * @return The JSON representation as a String
	 */
	private String serializeVoid(Void pVoid) {
		return serializeNull(pVoid);
	}

	/**
	 * Serializes a primitive object into JSON
	 * 
	 * @param pObj The object to be JSON Serialized
	 * @return The JSON representation as a String
	 */
	private String serializePrimitive(Object pObj) {		
		if(pObj==null) return serializeNull(pObj); 
		if(pObj instanceof Boolean) return serializeBoolean((Boolean)pObj);
		if(pObj instanceof Byte) return serializeByte((Byte)pObj);
		if(pObj instanceof Character) return serializeCharacter((Character)pObj);
		if(pObj instanceof Double) return serializeDouble((Double)pObj);
		if(pObj instanceof Enum) return serializeEnum((Enum)pObj);
		if(pObj instanceof Float) return serializeFloat((Float)pObj);
		if(pObj instanceof Integer) return serializeInteger((Integer)pObj);
		if(pObj instanceof Long) return serializeLong((Long)pObj);
		if(pObj instanceof String) return serializeString((String)pObj);
		if(pObj instanceof Short) return serializeShort((Short)pObj);
		if(pObj instanceof Void) return serializeVoid((Void)pObj);
		//Generic Number
		if(pObj instanceof Number) return serializeNumber((Number)pObj);
		return serializeNull(pObj);
	}

	/**
	 * Serializes an array of objects into JSON
	 * 
	 * @param pArr The array to be JSON Serialized
	 * @return The JSON representation as a String
	 */
	private String serializeArray(Object []pArr) {
		int mIter;	
		StringBuffer mStrb=new StringBuffer();
		if(pArr==null) return serializeNull(pArr);
		//Iterate
		mStrb.append(START_ARRAY);
		for(mIter=0;mIter<pArr.length;++mIter){
			mStrb.append(serializeObject(pArr[mIter]));
			if(mIter<(pArr.length-1)) mStrb.append(NEXT_ELEMENT);
		}
		mStrb.append(END_ARRAY);					
		return mStrb.toString();
	}

	/**
	 * Serializes a <code>Map</code> of objects into JSON
	 * 
	 * @param pMap The Map to be JSON Serialized
	 * @return The JSON representation as a String
	 */
	private String serializeMap(Map pMap) {
		Iterator mIterator = null;
		Object mKey=null;
        Object mObj=null;		
		StringBuffer mStrb=new StringBuffer();
		if(pMap==null) return serializeNull(pMap);
		//Iterate
		mIterator=pMap.keySet().iterator();
		if(mIterator==null) return serializeNull(pMap);
		mStrb.append(START_OBJECT);
		if(mIterator.hasNext()){			
            mKey = mIterator.next();            
            mObj=pMap.get(mKey);
            if(mKey==null||!(mKey instanceof String)) mStrb.append(serializeWrapper(null,mObj,false));
            else mStrb.append(serializeWrapper((String)mKey,mObj,false));            
			while (mIterator.hasNext()) {				
				mStrb.append(NEXT_ELEMENT);
	            mKey = mIterator.next();            
	            mObj=pMap.get(mKey);
	            if(mKey==null||!(mKey instanceof String)) mStrb.append(serializeWrapper(null,mObj,false));	            
	            else mStrb.append(serializeWrapper((String)mKey,mObj,false));
	        }	
		}		
		mStrb.append(END_OBJECT);					
		return mStrb.toString();
	}

	/**
	 * Serializes an <code>Iterable</code> collection of objects into JSON
	 * 
	 * @param pIter The Iterable collection to be JSON Serialized
	 * @return The JSON representation as a String
	 */
	private String serializeIterable(Iterable pIter) {
		Iterator mIterator = null;
		Object mObj=null;		
		StringBuffer mStrb=new StringBuffer();
		if(pIter==null) return serializeNull(pIter);
		//Iterate
		mIterator=pIter.iterator();
		if(mIterator==null) return serializeNull(pIter);
		mStrb.append(START_ARRAY);
		if(mIterator.hasNext()){			
            mObj = mIterator.next();                       
            mStrb.append(serializeObject(mObj));            
			while (mIterator.hasNext()) {				
				mStrb.append(NEXT_ELEMENT);
	            mObj = mIterator.next();                       
	            mStrb.append(serializeObject(mObj));
	        }	
		}
		mStrb.append(END_ARRAY);					
		return mStrb.toString();		
	}

	
	/**
	 * Serializes a composite object
	 * 
	 * @param pObj The Object to be JSON serialized 
	 * @return The JSON representation as a String
	 */
	private String serializeCompositeObject(Object pObj) {		
		StringBuffer mStrb=new StringBuffer();
		int mIter;
		Object mMemberObj=null;
		Class mClass=null;
		Method mMethod=null;
		Field []mFields=null;
		BeanInfo mBI=null;
		PropertyDescriptor[] mProps=null;
		Map mMap=new HashMap();
		
		if(pObj==null) return serializeNull(pObj);
		if(!isSerializable(pObj)) return serializeNull(pObj);
		
		mClass=pObj.getClass();
		if(mClass==null) return serializeNull(pObj);
		try {
			mBI=Introspector.getBeanInfo(mClass);
		} catch (IntrospectionException eIE) {
			eIE.printStackTrace();
			mBI=null;
		}	
		//Iterate		
		if(mBI==null); 
		else {
			mProps=mBI.getPropertyDescriptors();
			if(mProps!=null)
			for (mIter=0;mIter<mProps.length;++mIter) {
				mMemberObj=null;
				mMethod=mProps[mIter].getReadMethod();
				try {
					mMemberObj=mMethod.invoke(pObj, (Object [])null);
				} catch (IllegalArgumentException eIArE) {
					eIArE.printStackTrace();
					mMemberObj=null;
				} catch (IllegalAccessException eIAcE) {
					eIAcE.printStackTrace();
					mMemberObj=null;
				} catch (InvocationTargetException eITE) {
					eITE.printStackTrace();
					mMemberObj=null;
				}
				//Filter
				if(isSerializable(mMemberObj))	mMap.put(mProps[mIter].getName(), mMemberObj);				
			}
		}
		//Iterate
		mFields=mClass.getFields();
		if(mFields==null);
		else {
			for(mIter=0;mIter<mFields.length;++mIter){
				mMemberObj=null;
				try {
					mMemberObj=mFields[mIter].get(pObj);
				} catch (IllegalArgumentException eIArE) {
					eIArE.printStackTrace();
					mMemberObj=null;
				} catch (IllegalAccessException eIAcE) {
					eIAcE.printStackTrace();
					mMemberObj=null;
				}
				//Filter
				if(isSerializable(mMemberObj))	mMap.put(mProps[mIter].getName(), mMemberObj);				
			}		
		}
		mStrb.append(serializeMap(mMap));
		mMap.clear(); mMap=null;
		return mStrb.toString();
	}
	
	/**
	 * Serializes an object
	 * 
	 * @param pObj The Object to be JSON serialized 
	 * @return The JSON representation as a String
	 */
	private String serializeObject(Object pObj) {				
		if(pObj==null) return serializeNull(pObj);
		if(isPrimitive(pObj)) return serializePrimitive(pObj);
		if(isArray(pObj)) return serializeArray((Object[])pObj);
		if(pObj instanceof Map) return serializeMap((Map)pObj);
		if(pObj instanceof Iterable) return serializeIterable((Iterable)pObj);
		return serializeCompositeObject(pObj);			
	}
		
	/**
	 * A wrapper to serialize an object with it's name in JSON.
	 * This wrapper is used as a way to avoid unnecessary bracing
	 * of elements in JSON.
	 * 
	 * @param pName The JSON name of the object
	 * @param pObj The Object to be JSON serialized
	 * @param pEnclosed true if the serialized JSON representation is enclosed by braces.  
	 * @return The JSON representation as a String
	 */
	private String serializeWrapper(String pName, Object pObj, boolean pEnclosed) {
		StringBuffer mStrb=new StringBuffer();
		if(pEnclosed) mStrb.append(START_OBJECT);
		if(pName==null||"".equals(pName)) {
			mStrb.append(JS_VAR);
			mStrb.append(vCounter++);
		} else
			mStrb.append(serializeObject(jsVarQuoteString(pName)));
		mStrb.append(ASSIGN_ELEMENT);
		mStrb.append(serializeObject(pObj));
		if(pEnclosed) mStrb.append(END_OBJECT);
		return mStrb.toString();
	}
	
	/** 
	 * This function returns the JSON representation of an object
	 * as a <code>String</code>. It is the only way to use this 
	 * class. It calls the other functions internally to serialize any 
	 * given object/array/collection/composite into JSON. 
	 * 
	 * @param pName Name of the object passed 
	 * @param pObj The object passed 
	 * @return The JSON <code>String</code> representation of the object passed 
	 */
	public String serialize(String pName, Object pObj) {
		return serializeWrapper(pName,pObj,true);
	}
	
}